import { isVue2, markRaw } from 'vue-demi';
import { createModuleLoader } from './core/moduleLoader';
import { createComponentLoader } from './core/componentLoader';
import { ModuleLoaderSymbol, setActiveLoader, setModuleLoaderOptions, addErrorHandler, removeErrorHandler, } from './register';
export function createLoader() {
    var loader = markRaw({
        install: function (app) {
            // this allows calling registerSubModules() outside of a component setup after
            setActiveLoader(loader);
            if (!isVue2) {
                var moduleLoader = createModuleLoader(app);
                var componentLoader = createComponentLoader(app);
                loader._a = app;
                loader._moduleLoader = moduleLoader;
                app.config.globalProperties.$componentLoader = loader._componentLoader = componentLoader;
                app.provide(ModuleLoaderSymbol, loader);
            }
        },
        setOptions: function (options) {
            setModuleLoaderOptions.call(null, options);
            return this;
        },
        addErrorHandler: function (handler) {
            addErrorHandler.call(null, handler);
            return this;
        },
        removeErrorHandler: function (handler) {
            removeErrorHandler.call(null, handler);
            return this;
        },
        // it's actually undefined here
        // @ts-expect-error set in install when using Vue 3
        _a: null,
    });
    return loader;
}
